# -*- coding: utf-8 -*-
from setuphelpers import *
import os
import sys

libdev_dir = os.sep.join([sys.executable.rsplit(os.path.sep, 1)[0].replace("/bin", ""), "libdev"])
libdev_sp_dir = os.sep.join([sys.executable.rsplit(os.path.sep, 1)[0].replace("/bin", ""), "libdev", "site-packages"])
sys.path.insert(0, libdev_dir)
sys.path.insert(0, libdev_sp_dir)


package_name = "icoextract"
python_version = "".join(str(x) for x in sys.version_info[:2])


def update_package():
    # Declaring local variables
    package_updated = False
    proxies = get_proxies()
    if not proxies:
        proxies = get_proxies_from_wapt_console()
    api_url = f"https://pypi.org/pypi/{package_name}/json"
    index_url = None
    # index_url = "https://pip.ad.tranquil.it/"
    # fixed_version = "9.5.0"
    fixed_version = None

    # Getting latest version information from official sources
    print("API used is: %s" % api_url)
    json_load = wgets(api_url, proxies=proxies, as_json=True)
    if fixed_version:
        json_search = json_load["releases"][fixed_version]
        version = fixed_version
    else:
        json_search = json_load["urls"]
        version = json_load["info"]["version"]
    for to_download in json_search:
        filename = to_download["filename"]
        if ".tar.gz" in filename:
            download_url = to_download["url"]
            latest_bin = package_name + os.sep + filename
            if not isfile(latest_bin):
                print("Downloading: %s" % latest_bin)
                wget(download_url, latest_bin, proxies=proxies)
            else:
                print("Binary is present: %s" % latest_bin)

    # Deleting outdated binaries
    for f in glob.glob("*.whl"):
        print("Removing: %s" % f)
        remove_file(f)

    # Deleting other binaries
    for f in glob.glob(f'{package_name}/{json_load["info"]["name"]}*.whl'):  # filename.split("-")[0]
        if not version in f:
            print("Removing: %s" % f)
            remove_file(f)

    # Deleting other dirs
    for dir_name in glob.glob("*/"):
        dir_name = dir_name.split(os.sep)[0]
        if not dir_name in ["WAPT", "__pycache__"] and not package_name in dir_name:
            remove_tree(dir_name)

    # generate_requirements_txt
    if not isfile("requirements.txt"):
        generate_requirements_txt(package_name, glob.glob(f"{package_name}/*.whl"))
    else:
        with open("requirements.txt") as f:
            requirements_list = f.read().split("\n")
            requirements_list = [a.split("==")[0] for a in requirements_list]
        for package in glob.glob(f"{package_name}/*.whl"):
            package_info = package.split(os.sep)[-1].split("-")
            name = package_info[0].replace("_", "-").lower()
            if not name in requirements_list:
                remove_file(package)

    # control autocompletion
    api_url = f"https://pypi.org/pypi/{package_name}/json"
    json_load = wgets(api_url, proxies=proxies, as_json=True)
    control.package = control.package.split("-", 1)[0] + "-waptpython-" + json_load["info"]["name"].lower()
    control.name = "waptpython " + json_load["info"]["name"]
    control.installed_size = get_size(package_name) * 3
    if json_load["info"].get("keywords") is not None and json_load["info"]["keywords"]:
        print(f'You may wanna add: "{json_load["info"]["keywords"].lower()}" in: "control.keywords"')
    control.description = json_load["info"]["summary"]
    if json_load["info"]["project_urls"].get("Changelog") is not None and json_load["info"]["project_urls"]["Changelog"]:
        control.changelog = json_load["info"]["project_urls"]["Changelog"]
    control.sources = json_load["info"]["package_url"] + "#files"
    if json_load["info"]["project_urls"].get("Homepage") is not None and json_load["info"]["project_urls"]["Homepage"]:
        control.homepage = json_load["info"]["project_urls"]["Homepage"]

    # Changing version of the package
    if Version(version, 4) > Version(control.get_software_version(), 4):
        print("Software version updated (from: %s to: %s)" % (control.get_software_version(), Version(version)))
        package_updated = True
    else:
        print("Software version up-to-date (%s)" % Version(version))
    control.set_software_version(version)
    control.save_control_to_wapt()

    # Validating update-package-sources
    return package_updated

    # # Changing version of the package and validating update-package-sources
    # return complete_control_version(control, version)


def generate_requirements_txt(package_name, packages_list):
    latest_versions = {}

    for package in packages_list:
        package_info = package.split(os.sep)[-1].split("-")
        name = package_info[0].replace("_", "-").lower()
        version = package_info[1]

        if name not in latest_versions:
            latest_versions[name] = version
        else:
            if version > latest_versions[name]:
                latest_versions[name] = version

    # if name == package_name:
    #     requirements_content = [f"{name}=={version}" for name, version in latest_versions.items()]
    # else:
    #     requirements_content = [f"{name}" for name, version in latest_versions.items()]
    requirements_content = [f"{name}" for name, version in latest_versions.items()]
    if requirements_content:
        with open("requirements.txt", "w", newline="\n") as requirements_file:
            requirements_file.write("\n".join(requirements_content))


def get_waptpython_path():
    if get_os_name() == "Windows":
        return makepath(WAPT.wapt_base_dir, "waptpython.exe")
    else:
        return makepath(WAPT.wapt_base_dir, "waptpython.sh")


def get_size(start_path="."):
    total_size = 0
    for dirpath, dirnames, filenames in os.walk(start_path):
        for f in filenames:
            fp = os.path.join(dirpath, f)
            # skip if it is symbolic link
            if not os.path.islink(fp):
                total_size += os.path.getsize(fp)
    return total_size
